@@ -57,11 +57,11 @@ init_startup_state (GladeXML *dialog)
 		if (!at_startup_state.enabled.osk_installed &&
 		    !(at_startup_state.enabled.screenreader_installed ||
 		      at_startup_state.enabled.magnifier_installed)) {
-			warning_label = g_strdup_printf ("<i>%s</i>", _("No Assistive Technology is available on your system.  The 'gok' package must be installed in order to get on-screen keyboard support, and the 'gnopernicus' package must be installed for screenreading and magnifying capabilities."));
+			warning_label = g_strdup_printf ("<i>%s</i>", _("No Assistive Technology is available on your system.  The 'gok' package must be installed in order to get on-screen keyboard support, and the 'orca' package must be installed for screenreading and magnifying capabilities."));
 		} else if (!at_startup_state.enabled.osk_installed) {
 			warning_label = g_strdup_printf ("<i>%s</i>", _("Not all available assistive technologies are installed on your system.  The 'gok' package must be installed in order to get on-screen keyboard support."));
 		} else {
-			warning_label = g_strdup_printf ("<i>%s</i>", _("Not all available assistive technologies are installed on your system.  The 'gnopernicus' package must be installed for screenreading and magnifying capabilities."));
+			warning_label = g_strdup_printf ("<i>%s</i>", _("Not all available assistive technologies are installed on your system.  The 'orca' package must be installed for screenreading and magnifying capabilities."));
 		}
 		gtk_label_set_markup (GTK_LABEL (WID ("at_applications_warning_label")), warning_label);
 		g_free (warning_label);
@@ -72,18 +72,20 @@ static GladeXML *
 create_dialog (void)
 {
 	GladeXML *dialog;
-	
+
 	dialog = glade_xml_new (GLADEDIR "/at-enable-dialog.glade", "at_properties_dialog", NULL);
-	
-	gtk_image_set_from_stock (GTK_IMAGE (WID ("at_close_and_logout_image")),
-				  GTK_STOCK_QUIT, GTK_ICON_SIZE_BUTTON);
-	
-	gtk_image_set_from_file (GTK_IMAGE (WID ("at_enable_image")),
-				 PIXMAPDIR "/at-support.png");
-	
-	gtk_image_set_from_file (GTK_IMAGE (WID ("at_applications_image")),
-				 PIXMAPDIR "/at-startup.png");
-	
+
+	if (dialog) {
+		gtk_image_set_from_stock (GTK_IMAGE (WID ("at_close_and_logout_image")),
+					  GTK_STOCK_QUIT, GTK_ICON_SIZE_BUTTON);
+
+		gtk_image_set_from_file (GTK_IMAGE (WID ("at_enable_image")),
+					 PIXMAPDIR "/at-support.png");
+
+		gtk_image_set_from_file (GTK_IMAGE (WID ("at_applications_image")),
+					 PIXMAPDIR "/at-startup.png");
+	}
+
 	return dialog;
 }
 
@@ -271,26 +273,33 @@ setup_dialog (GladeXML *dialog)
 int
 main (int argc, char *argv[])
 {
+	GnomeProgram *program;
 	GladeXML *dialog;
 	
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 	
-	gnome_program_init ("gnome-at-properties", VERSION,
-			    LIBGNOMEUI_MODULE, argc, argv,
-			    GNOME_PARAM_APP_DATADIR, GNOMECC_DATA_DIR,
-			    NULL);
+	program = gnome_program_init ("gnome-at-properties", VERSION,
+				      LIBGNOMEUI_MODULE, argc, argv,
+				      GNOME_PARAM_APP_DATADIR, GNOMECC_DATA_DIR,
+				      NULL);
 	
 	activate_settings_daemon ();
 	
 	dialog = create_dialog ();
-	
-	init_startup_state (dialog);
-	
-	setup_dialog (dialog);
 
-	gtk_main ();
-	
+	if (dialog) {
+		init_startup_state (dialog);
+
+		setup_dialog (dialog);
+
+		gtk_main ();
+
+		g_object_unref (dialog);
+	}
+
+	g_object_unref (program);
+
 	return 0;
 }
