@@ -18,7 +18,7 @@ enum {
 
 #define REVERT_COUNT 20
 
-struct {
+static struct {
 	Rotation  rotation;
 	gchar const    * name;
 } const rotations[] = {
@@ -314,7 +314,7 @@ wrap_in_label (GtkWidget *child, char *text)
   vbox = gtk_vbox_new (FALSE, 6);
   label = NULL;
 
-  label = gtk_label_new ("");
+  label = gtk_label_new (NULL);
 
   str = g_strdup_printf ("<b>%s</b>", text);
   gtk_label_set_markup (GTK_LABEL (label), str);
@@ -666,10 +666,12 @@ create_dialog (struct DisplayInfo *info)
       resolution = gconf_client_get_string (client, key, NULL);
       g_free (key);
       g_object_unref (client);
+
+      info->was_per_computer = resolution != NULL;
+      g_free (resolution);
       
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (per_computer_check),
-				    resolution != NULL);
-      info->was_per_computer = resolution != NULL;
+				    info->was_per_computer);
       
       gtk_widget_show (per_computer_check);
       
@@ -677,8 +679,6 @@ create_dialog (struct DisplayInfo *info)
       gtk_box_pack_start (GTK_BOX (vbox),
 			  wrapped, FALSE, FALSE, 0);
       gtk_widget_show (wrapped);
-
-      g_free (resolution);
     }
 
   info->per_computer_check = per_computer_check;
