@@ -11,7 +11,6 @@
 #include <glade/glade.h>
 #include <X11/Xatom.h>
 
-#include "gnome-theme-info.h"
 #include "wm-common.h"
 #include "capplet-util.h"
 #include "eggcellrendererkeys.h"
@@ -102,6 +101,7 @@ static const KeyListEntry metacity_key_list[] =
   { "/apps/metacity/global_keybindings/switch_windows",            ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/global_keybindings/switch_panels",             ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/global_keybindings/cycle_windows",             ALWAYS_VISIBLE,  0 },
+  { "/apps/metacity/global_keybindings/cycle_group",               ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/global_keybindings/cycle_panels",              ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/global_keybindings/show_desktop",              ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/global_keybindings/switch_to_workspace_1",     N_WORKSPACES_GT, 1 },
@@ -170,7 +170,7 @@ create_dialog (void)
 {
   GladeXML *dialog;
 
-  dialog = glade_xml_new (GNOMECC_DATA_DIR "/interfaces/gnome-keybinding-properties.glade", "gnome-keybinding-dialog", NULL);
+  dialog = glade_xml_new (GNOMECC_GLADE_DIR "/gnome-keybinding-properties.glade", "gnome-keybinding-dialog", NULL);
 
   return dialog;
 }
@@ -182,7 +182,9 @@ binding_name (guint                   keyval,
               gboolean                translate)
 {
   if (keyval != 0 || keycode != 0)
-    return egg_virtual_accelerator_name (keyval, keycode, mask);
+    return translate ?
+	egg_virtual_accelerator_label (keyval, keycode, mask) :
+	egg_virtual_accelerator_name (keyval, keycode, mask);
   else
     return translate ? g_strdup (_("Disabled")) : g_strdup ("disabled");
 }
@@ -297,12 +299,8 @@ keyentry_sort_func (GtkTreeModel *model,
 {
   KeyEntry *key_entry_a;
   KeyEntry *key_entry_b;
-  char *name_a;
-  char *name_b;
   int retval;
 
-  /* mmmmm, super-slow. */
-
   key_entry_a = NULL;
   gtk_tree_model_get (model, a,
                       KEYENTRY_COLUMN, &key_entry_a,
@@ -313,91 +311,110 @@ keyentry_sort_func (GtkTreeModel *model,
                       KEYENTRY_COLUMN, &key_entry_b,
                       -1);
 
-  
-  if (key_entry_a != NULL)
-    name_a = binding_name (key_entry_a->keyval,
-		    	   key_entry_a->keycode,
-                           key_entry_a->mask,
-                           TRUE);
-  else
-    name_a = NULL;
+  if (key_entry_a && key_entry_b)
+    {
+      if ((key_entry_a->keyval || key_entry_a->keycode) &&
+          (key_entry_b->keyval || key_entry_b->keycode))
+        {
+          gchar *name_a, *name_b;
 
-  if (key_entry_b != NULL)
-    name_b = binding_name (key_entry_b->keyval,
-		    	   key_entry_b->keycode,
-                           key_entry_b->mask,
-                           TRUE);
-  else
-    name_b = NULL;
+          name_a = binding_name (key_entry_a->keyval,
+                                 key_entry_a->keycode,
+                                 key_entry_a->mask,
+                                 TRUE);
 
-  if (name_a && name_b)
-    retval = g_utf8_collate (name_a, name_b);
-  else if (name_a)
-    retval = 1;
-  else if (name_b)
+          name_b = binding_name (key_entry_b->keyval,
+                                 key_entry_b->keycode,
+                                 key_entry_b->mask,
+                                 TRUE);
+
+          retval = g_utf8_collate (name_a, name_b);
+
+          g_free (name_a);
+          g_free (name_b);
+        }
+      else if (key_entry_a->keyval || key_entry_a->keycode)
+        retval = -1;
+      else if (key_entry_b->keyval || key_entry_b->keycode)
+        retval = 1;
+      else
+        retval = 0;
+    }
+  else if (key_entry_a)
     retval = -1;
+  else if (key_entry_b)
+    retval = 1;
   else
     retval = 0;
 
-  g_free (name_a);
-  g_free (name_b);
-
   return retval;
 }
 
 static void
-clear_old_model (GladeXML  *dialog,
-		 GtkWidget *tree_view)
+clear_old_model (GladeXML  *dialog)
 {
+  GtkWidget *tree_view;
   GtkTreeModel *model;
-  GtkTreeModel *sort_model;
-  GtkTreeIter iter;
-  KeyEntry *key_entry;
-  gboolean valid;
-  GConfClient *client;
 
-  client = gconf_client_get_default ();
+  tree_view = WID ("shortcut_treeview");
   model = get_real_model (GTK_TREE_VIEW (tree_view));
 
-  if (model != NULL)
+  if (model == NULL)
     {
+      /* create a new model */
+      GtkTreeModel *sort_model;
+
+      model = (GtkTreeModel *) gtk_tree_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_POINTER);
+  
+      sort_model = gtk_tree_model_sort_new_with_model (model);
+
+      gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (sort_model),
+                                       KEYENTRY_COLUMN,
+                                       keyentry_sort_func,
+                                       NULL, NULL);
+  
+      gtk_tree_view_set_model (GTK_TREE_VIEW (tree_view), sort_model);
+
+      g_object_unref (model);
+      g_object_unref (sort_model);
+    }
+  else
+    {
+      /* clear the existing model */
+      GConfClient *client;
+      gboolean valid;
+      GtkTreeIter iter;
+      KeyEntry *key_entry;
+
+      client = gconf_client_get_default ();
+      /* we need the schema name below;
+       * cached values do not have that set, though */
+      gconf_client_clear_cache (client);
+
       g_object_ref (model);
 
       for (valid = gtk_tree_model_get_iter_first (model, &iter);
-	   valid;
-	   valid = gtk_tree_model_iter_next (model, &iter))
-	{
-	  gtk_tree_model_get (model, &iter,
-			      KEYENTRY_COLUMN, &key_entry,
-			      -1);
-	  if (key_entry != NULL)
-	    {
-	      gconf_client_notify_remove (client, key_entry->gconf_cnxn);
-	      g_free (key_entry->gconf_key);
+           valid;
+           valid = gtk_tree_model_iter_next (model, &iter))
+        {
+          gtk_tree_model_get (model, &iter,
+                              KEYENTRY_COLUMN, &key_entry,
+                              -1);
+
+          if (key_entry != NULL)
+            {
+              gconf_client_notify_remove (client, key_entry->gconf_cnxn);
+              g_free (key_entry->gconf_key);
               g_free (key_entry->description);
-	      g_free (key_entry);
+              g_free (key_entry);
             }
-	}
-      g_object_unref (model);
+        }
+
+      gtk_tree_store_clear(GTK_TREE_STORE(model));
+   	  g_object_unref (model);
+      g_object_unref (client);
     }
 
-  g_object_unref (client);
-
-  model = (GtkTreeModel *) gtk_tree_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_POINTER);
-  
-  sort_model = gtk_tree_model_sort_new_with_model (model);
-
-  /* N_COLUMNS is just a place to stick the extra sort function */
-  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (sort_model),
-                                   N_COLUMNS,
-                                   keyentry_sort_func,
-                                   NULL, NULL);
-  
-  gtk_tree_view_set_model (GTK_TREE_VIEW (tree_view), sort_model);
-
-  g_object_unref (G_OBJECT (model));
-  g_object_unref (G_OBJECT (sort_model));
-  
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (WID ("actions_swindow")),
 				  GTK_POLICY_NEVER, GTK_POLICY_NEVER);
   gtk_widget_set_usize (WID ("actions_swindow"), -1, -1);
@@ -550,7 +567,7 @@ append_keys_to_tree (GladeXML           *dialog,
 static void
 reload_key_entries (gpointer wm_name, GladeXML *dialog)
 {
-  clear_old_model (dialog, WID ("shortcut_treeview"));
+  clear_old_model (dialog);
   
   append_keys_to_tree (dialog, _("Desktop"), desktop_key_list);
   append_keys_to_tree (dialog, _("Sound"), sounds_key_list);
@@ -567,7 +584,9 @@ key_entry_controlling_key_changed (GConfClient *client,
 				   GConfEntry  *entry,
 				   gpointer     user_data)
 {
-  reload_key_entries (wm_common_get_current_window_manager(), user_data);
+  gchar *wm_name = wm_common_get_current_window_manager();
+  reload_key_entries (wm_name, user_data);
+  g_free (wm_name);
 }
 
 static gboolean
@@ -627,6 +646,9 @@ accel_edited_callback (GtkCellRendererText   *cell,
   if (key_entry == NULL)
     return;
 
+  /* CapsLock isn't supported as a keybinding modifier, so keep it from confusing us */
+  mask &= ~EGG_VIRTUAL_LOCK_MASK;
+
   model = get_real_model (view);
   tmp_key.model  = model;
   tmp_key.keyval = keyval;
@@ -646,24 +668,28 @@ accel_edited_callback (GtkCellRendererText   *cell,
     {
       if ((tmp_key.keyval >= GDK_a && tmp_key.keyval <= GDK_z)
 	   || (tmp_key.keyval >= GDK_A && tmp_key.keyval <= GDK_Z)
-	   || (tmp_key.keyval >= GDK_kana_fullstop && tmp_key.keyval <= GDK_kana_switch)
-	   || (tmp_key.keyval >= GDK_Arabic_comma && tmp_key.keyval <= GDK_Arabic_switch)
+	   || (tmp_key.keyval >= GDK_0 && tmp_key.keyval <= GDK_9)
+	   || (tmp_key.keyval >= GDK_kana_fullstop && tmp_key.keyval <= GDK_semivoicedsound)
+	   || (tmp_key.keyval >= GDK_Arabic_comma && tmp_key.keyval <= GDK_Arabic_sukun)
 	   || (tmp_key.keyval >= GDK_Serbian_dje && tmp_key.keyval <= GDK_Cyrillic_HARDSIGN)
-	   || (tmp_key.keyval >= GDK_Greek_ALPHAaccent && tmp_key.keyval <= GDK_Greek_switch)
-	   || (tmp_key.keyval >= GDK_hebrew_doublelowline && tmp_key.keyval <= GDK_Hebrew_switch)
+	   || (tmp_key.keyval >= GDK_Greek_ALPHAaccent && tmp_key.keyval <= GDK_Greek_omega)
+	   || (tmp_key.keyval >= GDK_hebrew_doublelowline && tmp_key.keyval <= GDK_hebrew_taf)
 	   || (tmp_key.keyval >= GDK_Thai_kokai && tmp_key.keyval <= GDK_Thai_lekkao)
-	   || (tmp_key.keyval >= GDK_Hangul && tmp_key.keyval <= GDK_Hangul_J_YeorinHieuh)) {
+	   || (tmp_key.keyval >= GDK_Hangul && tmp_key.keyval <= GDK_Hangul_Special)
+	   || (tmp_key.keyval >= GDK_Hangul_Kiyeog && tmp_key.keyval <= GDK_Hangul_J_YeorinHieuh)
+	   || tmp_key.keyval == GDK_space || tmp_key.keyval == GDK_Return
+	   || tmp_key.keyval == GDK_Mode_switch) {
         GtkWidget *dialog;
 	char *name;
 
-	name = egg_virtual_accelerator_name (keyval, keycode, mask);
+	name = binding_name (keyval, keycode, mask, TRUE);
 
 	dialog =
 	  gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (view))),
 			  	  GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
 				  GTK_MESSAGE_WARNING,
 				  GTK_BUTTONS_CANCEL,
-				  _("The shortcut \"%s\" cannot be used because it will become unusable to type using this key.\n"
+				  _("The shortcut \"%s\" cannot be used because it will become impossible to type using this key.\n"
 				  "Please try with a key such as Control, Alt or Shift at the same time.\n"),
 				  name);
 
@@ -685,7 +711,7 @@ accel_edited_callback (GtkCellRendererText   *cell,
       GtkWidget *dialog;
       char *name;
 
-      name = egg_virtual_accelerator_name (keyval, keycode, mask);
+      name = binding_name (keyval, keycode, mask, TRUE);
       
       dialog =
         gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (view))),
@@ -857,11 +883,18 @@ static void
 cb_dialog_response (GtkWidget *widget, gint response_id, gpointer data)
 {
 	if (response_id == GTK_RESPONSE_HELP)
-		capplet_help (GTK_WINDOW (widget),
-			      "user-guide.xml",
-			      "goscustdesk-39");
+          {
+            capplet_help (GTK_WINDOW (widget),
+                          "user-guide.xml",
+                          "goscustdesk-39");
+          }
 	else
-		gtk_main_quit ();
+          {
+            GladeXML *dialog = data;
+
+            clear_old_model (dialog);
+            gtk_main_quit ();
+          }
 }
 
 static void
@@ -871,6 +904,7 @@ setup_dialog (GladeXML *dialog)
   GtkCellRenderer *renderer;
   GtkTreeViewColumn *column;
   GtkWidget *widget;
+  gchar *wm_name;
 
   client = gconf_client_get_default ();
 
@@ -910,8 +944,7 @@ setup_dialog (GladeXML *dialog)
   gtk_tree_view_column_set_resizable (column, FALSE);
 
   gtk_tree_view_append_column (GTK_TREE_VIEW (WID ("shortcut_treeview")), column);
-  /* N_COLUMNS is just a place to stick the extra sort function */
-  gtk_tree_view_column_set_sort_column_id (column, N_COLUMNS); 
+  gtk_tree_view_column_set_sort_column_id (column, KEYENTRY_COLUMN); 
   
   gconf_client_add_dir (client, "/apps/gnome_keybinding_properties", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
   gconf_client_add_dir (client, "/apps/metacity/general", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
@@ -922,19 +955,21 @@ setup_dialog (GladeXML *dialog)
   g_object_unref (client);
 
   /* set up the dialog */
-  reload_key_entries (wm_common_get_current_window_manager(), dialog);
+  wm_name = wm_common_get_current_window_manager();
+  reload_key_entries (wm_name, dialog);
+  g_free (wm_name);
 
   widget = WID ("gnome-keybinding-dialog");
   capplet_set_icon (widget, "gnome-settings-keybindings");
   gtk_widget_show (widget);
 
-  g_signal_connect (G_OBJECT (widget), "response", G_CALLBACK(cb_dialog_response), NULL);
-  g_signal_connect (G_OBJECT (widget), "close", gtk_main_quit, NULL);
+  g_signal_connect (G_OBJECT (widget), "response", G_CALLBACK(cb_dialog_response), dialog);
 }
 
 int
 main (int argc, char *argv[])
 {
+  GnomeProgram *program;
   GladeXML *dialog;
 
   gtk_init (&argc, &argv);
@@ -943,12 +978,11 @@ main (int argc, char *argv[])
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
-  gnome_program_init ("gnome-keybinding-properties", VERSION, LIBGNOMEUI_MODULE, argc, argv,
+  program = gnome_program_init ("gnome-keybinding-properties", VERSION,
+		      LIBGNOMEUI_MODULE, argc, argv,
 		      GNOME_PARAM_APP_DATADIR, GNOMECC_DATA_DIR,
 		      NULL);
 
-  gnome_theme_init (NULL);
-
   activate_settings_daemon ();
 
   dialog = create_dialog ();
@@ -957,5 +991,7 @@ main (int argc, char *argv[])
   
   gtk_main ();
 
+  g_object_unref (dialog);
+  g_object_unref (program);
   return 0;
 }
