@@ -156,28 +156,23 @@ bell_flash_to_widget (GConfPropertyEditor *peditor, const GConfValue *value)
 	return new_value;
 }
 
-/* create_dialog
- *
- * Create the dialog box and return it as a GtkWidget
- */
-
 static GladeXML *
 create_dialog (void) 
 {
 	GladeXML *dialog;
-	GtkWidget *widget, *box;
+	GtkWidget *widget, *box, *view;
 
-	dialog = glade_xml_new (GNOMECC_DATA_DIR "/interfaces/sound-properties.glade", "prefs_widget", NULL);
-	widget = glade_xml_get_widget (dialog, "prefs_widget");
-	g_object_set_data (G_OBJECT (widget), "glade-data", dialog);
+	dialog = glade_xml_new (GNOMECC_GLADE_DIR "/sound-properties.glade", "sound_prefs_dialog", NULL);
+	widget = glade_xml_get_widget (dialog, "sound_prefs_dialog");
 
 	props = sound_properties_new ();
 	sound_properties_add_defaults (props, NULL);
 	g_signal_connect (G_OBJECT (props), "event_changed",
-			  (GCallback) props_changed_cb, NULL);  
+			  (GCallback) props_changed_cb, NULL);
+	view = sound_view_new (props);
 	box = glade_xml_get_widget (dialog, "events_vbox");
-	gtk_box_pack_start (GTK_BOX (box), sound_view_new (props),
-			    TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (box), view, TRUE, TRUE, 0);
+	gtk_widget_show_all (view);
 
 	g_signal_connect_swapped (G_OBJECT (widget), "destroy",
 				  (GCallback) gtk_object_destroy, props);
@@ -185,14 +180,8 @@ create_dialog (void)
 	gtk_image_set_from_file (GTK_IMAGE (WID ("bell_image")),
 				 GNOMECC_DATA_DIR "/pixmaps/visual-bell.png");
 
-	gtk_widget_set_size_request (widget, 475, -1);
-
 	if (!CheckXKB()) {
-		GtkWidget *audible_bell_option = WID ("bell_audible_toggle");
-		GtkWidget *visual_bell_option = WID ("bell_visual_toggle");
-
-		gtk_widget_set_sensitive (audible_bell_option, FALSE);
-		gtk_widget_set_sensitive (visual_bell_option, FALSE);
+		gtk_widget_set_sensitive (WID ("bell_flash_alignment"), FALSE);
 	}
 		
 	return dialog;
@@ -452,7 +441,7 @@ device_removed_callback (LibHalContext *ctx, const char *udi)
 }
 
 static void
-setup_hal_devices ()
+setup_hal_devices (void)
 {
 	DBusConnection *connection;
 	DBusError error;
@@ -513,7 +502,7 @@ device_test_button_clicked (GtkWidget *button, gpointer user_data)
 {
 	DeviceChooser *device_chooser = (DeviceChooser *) user_data;
 	
-	GladeXML *dialog = glade_xml_new (GNOMECC_DATA_DIR "/interfaces/sound-properties.glade", NULL, NULL);
+	GladeXML *dialog = glade_xml_new (GNOMECC_GLADE_DIR "/sound-properties.glade", NULL, NULL);
 
 	user_test_pipeline (dialog, GTK_WINDOW (dialog_win), device_chooser->test_pipeline);
 	
@@ -652,7 +641,7 @@ add_selected_device (const gchar *profile, int type)
 }
 
 static void
-setup_devices ()
+setup_devices (void)
 {
 	add_device (AUDIO_PLAYBACK, "autoaudiosink", _("Autodetect"), NULL);
 
@@ -666,8 +655,8 @@ setup_devices ()
 	add_device (AUDIO_CAPTURE, "esdmon", _("ESD - Enlightened Sound Daemon"), NULL);
 	add_device (AUDIO_PLAYBACK, "osssink", _("OSS - Open Sound System"), NULL);
 	add_device (AUDIO_CAPTURE, "osssrc", _("OSS - Open Sound System"), NULL);
-	add_device (AUDIO_PLAYBACK, "polypsink", _("Polypaudio Sound Server"), NULL);
-	add_device (AUDIO_CAPTURE, "polypsrc", _("Polypaudio Sound Server"), NULL);
+	add_device (AUDIO_PLAYBACK, "pulsesink", _("PulseAudio Sound Server"), NULL);
+	add_device (AUDIO_CAPTURE, "pulsesrc", _("PulseAudio Sound Server"), NULL);
 	add_device (AUDIO_CAPTURE, "audiotestsrc wave=triangle is-live=true", _("Test Sound"), NULL);
 	add_device (AUDIO_CAPTURE, "audiotestsrc wave=silence is-live=true", _("Silence"), NULL);
 	
@@ -740,21 +729,23 @@ update_mixer_tracks_selection (GSList *tracks, GladeXML *dialog)
 
 	gtk_tree_selection_unselect_all (selection);
 
-	if (gtk_tree_model_get_iter_first (model, &iter)) {
-		do {
-			gchar *label;
-			GSList *t;
+	if (tracks != NULL) {
+		if (gtk_tree_model_get_iter_first (model, &iter)) {
+			do {
+				gchar *label;
+				GSList *t;
 
-			gtk_tree_model_get (model, &iter, MIXER_TRACKS_MODEL_LABEL_COLUMN, &label, -1);
+				gtk_tree_model_get (model, &iter, MIXER_TRACKS_MODEL_LABEL_COLUMN, &label, -1);
 
-			for (t = tracks; t != NULL; t = t->next) {
-				if (!strcmp (label, t->data)) {
-					gtk_tree_selection_select_iter (selection, &iter);
-					break;
+				for (t = tracks; t != NULL; t = t->next) {
+					if (!strcmp (label, t->data)) {
+						gtk_tree_selection_select_iter (selection, &iter);
+						break;
+					}
 				}
-			}
-			g_free (label);
-		} while (gtk_tree_model_iter_next (model, &iter));		
+				g_free (label);
+			} while (gtk_tree_model_iter_next (model, &iter));		
+		}
 	}
 		
 	g_signal_handlers_unblock_by_func (G_OBJECT (selection), G_CALLBACK (mixer_tracks_selection_changed), NULL);
@@ -786,26 +777,29 @@ update_mixer_device_combobox (const gchar *mixer_device, GladeXML *dialog)
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (device_widget));
 
 	/* try to find stored factory and device in the mixer device list */
-	if (gtk_tree_model_get_iter_first (model, &iter)) {
-		do {
-			gchar *device = NULL;
+	if (mixer_device != NULL && *mixer_device != '\0') {
+		if (gtk_tree_model_get_iter_first (model, &iter)) {
+			do {
+				gchar *device = NULL;
 
-			gtk_tree_model_get (model, &iter,
-					MIXER_DEVICE_MODEL_DEVICE_COLUMN, &device,
-					-1);
+				gtk_tree_model_get (model, &iter,
+						MIXER_DEVICE_MODEL_DEVICE_COLUMN, &device,
+						-1);
 
-			if (!strcmp (device, mixer_device)) {
-				gtk_combo_box_set_active_iter (GTK_COMBO_BOX (device_widget), &iter);
+				if (!strcmp (device, mixer_device)) {
+					gtk_combo_box_set_active_iter (GTK_COMBO_BOX (device_widget), &iter);
+
+					g_free (device);
+					break;
+				}
 
 				g_free (device);
-				break;
-			}
-
-			g_free (device);
-		} while (gtk_tree_model_iter_next (model, &iter));
+			} while (gtk_tree_model_iter_next (model, &iter));
+		}
 	}
 
 	/* try to select first mixer entry */
+	/* FIXME use the first with a master track */
 	if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (device_widget), &iter)) {
 		if (gtk_tree_model_get_iter_first (model, &iter)) {
 			gtk_combo_box_set_active_iter (GTK_COMBO_BOX (device_widget), &iter);
@@ -990,14 +984,16 @@ get_legacy_settings (void)
 }
 
 static void
-dialog_button_clicked_cb (GtkDialog *dialog, gint response_id, GConfChangeSet *changeset) 
+dialog_response_cb (GtkWidget *dialog, gint response_id, GConfChangeSet *changeset) 
 {
-	if (response_id == GTK_RESPONSE_HELP)
+	if (response_id == GTK_RESPONSE_HELP) {
 		capplet_help (GTK_WINDOW (dialog),
 			"user-guide.xml",
 			"goscustmulti-2");
-	else
-		gtk_main_quit ();
+		return;
+	}
+	
+	gtk_widget_destroy (dialog);
 }
 
 int
@@ -1005,11 +1001,11 @@ main (int argc, char **argv)
 {
 	GConfChangeSet *changeset;
 	GladeXML       *dialog = NULL;
+	GnomeProgram   *program;
  	GOptionContext *context;
-
-	static gboolean apply_only;
-	static gboolean get_legacy;
- 	static GOptionEntry cap_options[] = {
+	gboolean apply_only = FALSE;
+	gboolean get_legacy = FALSE;
+ 	GOptionEntry cap_options[] = {
  		{ "apply", 0, 0, G_OPTION_ARG_NONE, &apply_only,
 		  N_("Just apply settings and quit (compatibility only; now handled by daemon)"), NULL },
 		{ "init-session-settings", 0, 0, G_OPTION_ARG_NONE, &apply_only,
@@ -1025,13 +1021,12 @@ main (int argc, char **argv)
 
  	context = g_option_context_new (_("- GNOME Sound Preferences"));
  	g_option_context_add_main_entries (context, cap_options, GETTEXT_PACKAGE);
+	g_option_context_add_group (context, gst_init_get_option_group ());
 
-	gnome_program_init ("gnome-sound-properties", VERSION,
-			    LIBGNOMEUI_MODULE, argc, argv,
- 			    GNOME_PARAM_GOPTION_CONTEXT, context,
-			    NULL);
-
-	gst_init (&argc, &argv);
+	program = gnome_program_init ("gnome-sound-properties", VERSION,
+				      LIBGNOMEUI_MODULE, argc, argv,
+ 			    	      GNOME_PARAM_GOPTION_CONTEXT, context,
+				      NULL);
 
 	activate_settings_daemon ();
 	
@@ -1047,25 +1042,19 @@ main (int argc, char **argv)
 		setup_dialog (dialog, changeset);
 		setup_devices ();
 
-		dialog_win = gtk_dialog_new_with_buttons
-			(_("Sound Preferences"), NULL, GTK_DIALOG_NO_SEPARATOR,
-			 GTK_STOCK_HELP, GTK_RESPONSE_HELP,
-			 GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
-			 NULL);
-
-		gtk_container_set_border_width (GTK_CONTAINER (dialog_win), 5);
-		gtk_box_set_spacing (GTK_BOX (GTK_DIALOG(dialog_win)->vbox), 2);
-		gtk_dialog_set_default_response (GTK_DIALOG (dialog_win), GTK_RESPONSE_CLOSE);
-		g_signal_connect (G_OBJECT (dialog_win), "response", (GCallback) dialog_button_clicked_cb, changeset);
-		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_win)->vbox), WID ("prefs_widget"), TRUE, TRUE, 0);
+		dialog_win = WID ("sound_prefs_dialog");
+		g_signal_connect (dialog_win, "response", G_CALLBACK (dialog_response_cb), changeset);
+		g_signal_connect (dialog_win, "destroy", G_CALLBACK (gtk_main_quit), NULL);
 		capplet_set_icon (dialog_win, "gnome-settings-sound");
-		gtk_widget_show_all (dialog_win);
+		gtk_widget_show (dialog_win);
 
 		gtk_main ();
 		gconf_change_set_unref (changeset);
+		g_object_unref (dialog);
 	}
 	
 	g_object_unref (gconf_client);
-	g_object_unref (dialog);
+	g_object_unref (program);
+
 	return 0;
 }
