@@ -48,8 +48,11 @@ static void load_theme_names                (GtkTreeView        *tree_view,
 					     const gchar        *default_theme);
 static char *path_to_theme_id               (const char *path);
 static void update_color_buttons_from_string (gchar *color_scheme);
+static void color_scheme_changed            (GObject           *settings,
+                                             GParamSpec        *pspec,
+                                             gpointer           data);
 
-void toggle_color_scheme_key (GtkWidget *checkbutton, gpointer *data);
+void revert_color_scheme_key (GtkWidget *checkbutton, gpointer *data);
 
 static char *
 path_to_theme_id (const char *path)
@@ -203,20 +206,24 @@ gtk_theme_update_remove_button (GtkTreeSelection *selection,
 }
 
 static void
-update_color_scheme_tab ()
+update_color_scheme_tab (const gchar *theme)
 {
 	GSList *symbolic_colors = NULL;
 	GSList *engines = NULL;
 	gboolean fg, bg, base, text, fg_s, bg_s, enable_colors;
-	gchar *filename, *theme_name;
+	gchar *filename;
+	gchar *theme_name = NULL;
 	GtkSettings *settings;
 	GladeXML *dialog;
 
 	dialog = gnome_theme_manager_get_theme_dialog ();
 
-	settings = gtk_settings_get_default ();
-	g_object_get (G_OBJECT (settings), "gtk-theme-name", &theme_name, NULL);
-	filename = gtkrc_find_named (theme_name);
+	if (theme == NULL) {
+		settings = gtk_settings_get_default ();
+		g_object_get (G_OBJECT (settings), "gtk-theme-name", &theme_name, NULL);
+		theme = theme_name;
+	}
+	filename = gtkrc_find_named (theme);
 
 	gtkrc_get_details (filename, &engines, &symbolic_colors);
 	fg = (g_slist_find_custom (symbolic_colors, "fg_color", g_str_equal) != NULL);
@@ -227,23 +234,19 @@ update_color_scheme_tab ()
 	bg_s = (g_slist_find_custom (symbolic_colors, "selected_bg_color", g_str_equal) != NULL);
 
 	enable_colors = (fg && bg && base && text && fg_s && bg_s);
-	gtk_widget_set_sensitive (WID ("enable_custom_colors_checkbutton"), enable_colors);
+
+	gtk_widget_set_sensitive (WID ("color_scheme_table"), enable_colors);
+	gtk_widget_set_sensitive (WID ("color_scheme_revert_button"), enable_colors);
 
 	if (enable_colors)
-	{
-		gboolean enable_color_schemes;
-		enable_color_schemes = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (WID ("enable_custom_colors_checkbutton")));
-		gtk_widget_set_sensitive (WID ("color_scheme_table"), enable_color_schemes);
-		gtk_widget_set_sensitive (WID ("saved_color_schemes_hbox"), enable_color_schemes);
 		gtk_widget_hide (WID ("color_scheme_message_hbox"));
-	}
 	else
-	{
 		gtk_widget_show (WID ("color_scheme_message_hbox"));
-		gtk_widget_set_sensitive (WID ("color_scheme_table"), FALSE);
-		gtk_widget_set_sensitive (WID ("saved_color_schemes_hbox"), FALSE);
-	}
 
+	g_slist_foreach (engines, (GFunc) g_free, NULL);
+	g_slist_free (engines);
+	g_slist_foreach (symbolic_colors, (GFunc) g_free, NULL);
+	g_slist_free (symbolic_colors);
 	g_free (filename);
 	g_free (theme_name);
 }
@@ -253,11 +256,24 @@ gtk_theme_selection_changed (GtkTreeSelection *selection,
 			     gpointer          data)
 {
   GladeXML *dialog;
+  GtkTreeModel *model;
+  GtkTreeIter iter;
 
   dialog = gnome_theme_manager_get_theme_dialog ();
 
   update_gconf_key_from_selection (selection, GTK_THEME_KEY);
   gtk_theme_update_remove_button(selection, WID("control_remove_button"), THEME_GTK);
+
+  if (gtk_tree_selection_get_selected (selection, &model, &iter))
+    {
+      gchar *theme;
+
+      gtk_tree_model_get (model, &iter,
+                          THEME_ID_COLUMN, &theme,
+                          -1);
+      update_color_scheme_tab (theme);
+      g_free (theme);
+    }
 }
 
 static void
@@ -323,7 +339,7 @@ load_theme_names (GtkTreeView *tree_view,
 
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swindow),
 				  GTK_POLICY_NEVER, GTK_POLICY_NEVER);
-  gtk_widget_set_usize (swindow, -1, -1);
+  gtk_widget_set_size_request (swindow, -1, -1);
 
   for (list = theme_list; list; list = list->next->next)
     {
@@ -349,7 +365,7 @@ load_theme_names (GtkTreeView *tree_view,
 	{
 	  GtkRequisition rectangle;
 	  gtk_widget_size_request (GTK_WIDGET (tree_view), &rectangle);
-	  gtk_widget_set_usize (swindow, -1, rectangle.height);
+	  gtk_widget_set_size_request (swindow, -1, rectangle.height);
 	  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swindow),
 					  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	}
@@ -478,8 +494,8 @@ remove_theme(GtkWidget *button, gpointer data)
 	
 				uri_list = g_list_prepend(NULL, gnome_vfs_uri_new (theme_dir));
 
-				result = gnome_vfs_xfer_delete_list (uri_list, GNOME_VFS_XFER_RECURSIVE,
-								     GNOME_VFS_XFER_ERROR_MODE_ABORT,
+				result = gnome_vfs_xfer_delete_list (uri_list, GNOME_VFS_XFER_ERROR_MODE_ABORT,
+								     GNOME_VFS_XFER_RECURSIVE,
 								     NULL, NULL);
 				if (result == GNOME_VFS_OK) 
 				{
@@ -507,16 +523,14 @@ remove_theme(GtkWidget *button, gpointer data)
   }
 }
 
-void
-color_select (GtkWidget *colorbutton, GladeXML *dialog)
+static void
+gtk_color_scheme_changed (GtkWidget *colorbutton, GladeXML *dialog)
 {
   GConfClient *client = NULL;
   gchar *new_scheme;
   GdkColor colors[6];
   gchar *bg, *fg, *text, *base, *selected_fg, *selected_bg;
   GtkWidget *widget;
-  GtkTreeModel *model;
-  GtkTreeIter tmp, iter;
 
   widget = WID ("fg_colorbutton");
   gtk_color_button_get_color (GTK_COLOR_BUTTON (widget), &colors[0]);
@@ -531,13 +545,12 @@ color_select (GtkWidget *colorbutton, GladeXML *dialog)
   widget = WID ("selected_bg_colorbutton");
   gtk_color_button_get_color (GTK_COLOR_BUTTON (widget), &colors[5]);
 
-
-  fg = g_strdup_printf ("fg_color:#%.4x%.4x%.4x\n", colors[0].red, colors[0].green, colors[0].blue);
-  bg = g_strdup_printf ("bg_color:#%.4x%.4x%.4x\n", colors[1].red, colors[1].green, colors[1].blue);
-  text = g_strdup_printf ("text_color:#%.4x%.4x%.4x\n", colors[2].red, colors[2].green, colors[2].blue);
-  base = g_strdup_printf ("base_color:#%.4x%.4x%.4x\n", colors[3].red, colors[3].green, colors[3].blue);
-  selected_fg = g_strdup_printf ("selected_fg_color:#%.4x%.4x%.4x\n", colors[4].red, colors[4].green, colors[4].blue);
-  selected_bg = g_strdup_printf ("selected_bg_color:#%.4x%.4x%.4x", colors[5].red, colors[5].green, colors[5].blue);
+  fg = g_strdup_printf ("fg_color:#%04x%04x%04x\n", colors[0].red, colors[0].green, colors[0].blue);
+  bg = g_strdup_printf ("bg_color:#%04x%04x%04x\n", colors[1].red, colors[1].green, colors[1].blue);
+  text = g_strdup_printf ("text_color:#%04x%04x%04x\n", colors[2].red, colors[2].green, colors[2].blue);
+  base = g_strdup_printf ("base_color:#%04x%04x%04x\n", colors[3].red, colors[3].green, colors[3].blue);
+  selected_fg = g_strdup_printf ("selected_fg_color:#%04x%04x%04x\n", colors[4].red, colors[4].green, colors[4].blue);
+  selected_bg = g_strdup_printf ("selected_bg_color:#%04x%04x%04x", colors[5].red, colors[5].green, colors[5].blue);
 
   new_scheme = g_strconcat (fg, bg, text, base, selected_fg, selected_bg, NULL);
 
@@ -550,113 +563,19 @@ color_select (GtkWidget *colorbutton, GladeXML *dialog)
 
   g_object_unref (G_OBJECT (client));
   free_all (fg, bg, text, base, selected_fg, selected_bg, new_scheme, NULL);
-
-  model = gtk_combo_box_get_model (GTK_COMBO_BOX (WID ("color_scheme_combobox")));
-  for (gtk_tree_model_get_iter_first(GTK_TREE_MODEL (model), &tmp);gtk_tree_model_iter_next(model, &tmp);)
-  {
-    iter = tmp;
-  }
-  gtk_combo_box_set_active_iter (GTK_COMBO_BOX (WID ("color_scheme_combobox")), &iter);
-
 }
 
 void
-toggle_color_scheme_key (GtkWidget *checkbutton, gpointer *data)
+revert_color_scheme_key (GtkWidget *checkbutton, gpointer *data)
 {
   GConfClient *client = NULL;
-  gboolean use_custom_colors;
   GladeXML *dialog;
-  GtkSettings *settings;
-  gchar *color_scheme = NULL;
 
   dialog = gnome_theme_manager_get_theme_dialog ();
-  use_custom_colors = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton));
-
-  gtk_widget_set_sensitive (WID ("color_scheme_table"), use_custom_colors);
-  gtk_widget_set_sensitive (WID ("saved_color_schemes_hbox"), use_custom_colors);
-
-  if (!use_custom_colors)
-  {
-    client = gconf_client_get_default ();
-    gconf_client_set_string (client, COLOR_SCHEME_KEY, "", NULL);
-    g_object_unref (G_OBJECT (client));
-  }
-  else
-  {
-    settings = gtk_settings_get_default ();
-    g_object_get (G_OBJECT (settings), "gtk-color-scheme", &color_scheme, NULL);
-    update_color_buttons_from_string (color_scheme);
-    g_free (color_scheme);
-  }
-}
-
-void
-theme_notebook_changed_page (GtkWidget *widget, GladeXML *dialog)
-{
-	update_color_scheme_tab ();
-}
-
-void
-save_color_scheme (GtkWidget *widget, GtkWidget *parent)
-{
-	GtkWidget *dialog, *hbox, *label, *entry;
-	dialog = gtk_dialog_new_with_buttons (_("Save Color Scheme"),
-						GTK_WINDOW (parent),
-						GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
-						GTK_STOCK_CANCEL,
-						GTK_RESPONSE_CANCEL,
-						GTK_STOCK_SAVE,
-						GTK_RESPONSE_ACCEPT, NULL);
-
-	hbox = gtk_hbox_new (FALSE, 6);
-	label = gtk_label_new (_("Save color scheme as:"));
-	entry = gtk_entry_new ();
-
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 6);
-	gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, TRUE, 6);
-
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, TRUE, TRUE, 12);
-	gtk_widget_show_all (hbox);
-
-	gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_hide (dialog);
-}
-
-void
-delete_color_scheme (GtkWidget *widget, GtkWidget *parent)
-{
-	GladeXML *dialog;
-	gchar *color_scheme_name;
-	GtkWidget *query;
-	dialog = gnome_theme_manager_get_theme_dialog ();
-
-	color_scheme_name = gtk_combo_box_get_active_text (GTK_COMBO_BOX (WID ("color_scheme_combobox")));
-	query = gtk_message_dialog_new (GTK_WINDOW (parent),
-				GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-				GTK_MESSAGE_QUESTION,
-				GTK_BUTTONS_NONE,
-				_("Are you sure you want to delete the \"%s\" color scheme?"),
-				color_scheme_name);
-	gtk_dialog_add_buttons (GTK_DIALOG (query), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					GTK_STOCK_DELETE, GTK_RESPONSE_OK, NULL);
-	gtk_dialog_run (GTK_DIALOG (query));
-	gtk_widget_hide (query);
-}
-
-void
-color_scheme_combobox_changed (GtkWidget *widget, GtkWidget *parent)
-{
-	GladeXML *dialog;
-	dialog = gnome_theme_manager_get_theme_dialog ();
-	gchar *color_scheme_name;
-	gboolean custom;
-
-	color_scheme_name = gtk_combo_box_get_active_text (GTK_COMBO_BOX (WID ("color_scheme_combobox")));
-
-	custom = (color_scheme_name != NULL && strcmp (color_scheme_name, _("Custom")) == 0);
-	gtk_widget_set_sensitive (WID ("save_color_scheme_button"), custom);
-	gtk_widget_set_sensitive (WID ("delete_color_scheme_button"), !custom);
 
+  client = gconf_client_get_default ();
+  gconf_client_set_string (client, COLOR_SCHEME_KEY, "", NULL);
+  g_object_unref (G_OBJECT (client));
 }
 
 void
@@ -664,7 +583,7 @@ gnome_theme_details_init (void)
 {
   GtkWidget *parent, *widget;
   GladeXML *dialog;
-  gchar *color_scheme;
+  gchar *theme;
 
   if (theme_details_initted)
     return;
@@ -685,7 +604,7 @@ gnome_theme_details_init (void)
 
   /* gtk themes */
   widget = WID ("control_remove_button");
-  g_signal_connect (G_OBJECT (widget), "clicked", G_CALLBACK (remove_theme), THEME_GTK);
+  g_signal_connect (G_OBJECT (widget), "clicked", G_CALLBACK (remove_theme), (gint*)THEME_GTK);
   widget = WID ("control_install_button");
   g_signal_connect (G_OBJECT (widget), "clicked", G_CALLBACK (gnome_theme_installer_run_cb), parent);
 
@@ -703,34 +622,26 @@ gnome_theme_details_init (void)
 
   /* color preferences */
   widget = WID ("fg_colorbutton");
-  g_signal_connect (G_OBJECT (widget), "color_set", G_CALLBACK (color_select), dialog);
+  g_signal_connect (G_OBJECT (widget), "color_set", G_CALLBACK (gtk_color_scheme_changed), dialog);
   widget = WID ("bg_colorbutton");
-  g_signal_connect (G_OBJECT (widget), "color_set", G_CALLBACK (color_select), dialog);
+  g_signal_connect (G_OBJECT (widget), "color_set", G_CALLBACK (gtk_color_scheme_changed), dialog);
   widget = WID ("text_colorbutton");
-  g_signal_connect (G_OBJECT (widget), "color_set", G_CALLBACK (color_select), dialog);
+  g_signal_connect (G_OBJECT (widget), "color_set", G_CALLBACK (gtk_color_scheme_changed), dialog);
   widget = WID ("base_colorbutton");
-  g_signal_connect (G_OBJECT (widget), "color_set", G_CALLBACK (color_select), dialog);
+  g_signal_connect (G_OBJECT (widget), "color_set", G_CALLBACK (gtk_color_scheme_changed), dialog);
   widget = WID ("selected_fg_colorbutton");
-  g_signal_connect (G_OBJECT (widget), "color_set", G_CALLBACK (color_select), dialog);
+  g_signal_connect (G_OBJECT (widget), "color_set", G_CALLBACK (gtk_color_scheme_changed), dialog);
   widget = WID ("selected_bg_colorbutton");
-  g_signal_connect (G_OBJECT (widget), "color_set", G_CALLBACK (color_select), dialog);
+  g_signal_connect (G_OBJECT (widget), "color_set", G_CALLBACK (gtk_color_scheme_changed), dialog);
 
-  widget = WID ("enable_custom_colors_checkbutton");
-  g_signal_connect (G_OBJECT (widget), "toggled", G_CALLBACK (toggle_color_scheme_key), parent);
+  widget = WID ("color_scheme_revert_button");
+  g_signal_connect (G_OBJECT (widget), "clicked", G_CALLBACK (revert_color_scheme_key), parent);
 
-  widget = WID ("save_color_scheme_button");
-  g_signal_connect (G_OBJECT (widget), "clicked", G_CALLBACK (save_color_scheme), parent);
+  g_object_get (G_OBJECT (gtk_settings_get_default()), "gtk-color-scheme", &theme, NULL);
+  update_color_buttons_from_string (theme);
 
-  widget = WID ("color_scheme_combobox");
-  g_signal_connect (G_OBJECT (widget), "changed", G_CALLBACK (color_scheme_combobox_changed), parent);
-
-  widget = WID ("delete_color_scheme_button");
-  g_signal_connect (G_OBJECT (widget), "clicked", G_CALLBACK (delete_color_scheme), parent);
-
-  g_signal_connect (G_OBJECT (WID ("theme_notebook")), "switch-page", G_CALLBACK (theme_notebook_changed_page), dialog);
-
-  g_object_get (G_OBJECT (gtk_settings_get_default()), "gtk-color-scheme", &color_scheme, NULL);
-  update_color_buttons_from_string (color_scheme);
+  g_signal_connect (gtk_settings_get_default(), "notify::gtk-color-scheme", 
+                    G_CALLBACK (color_scheme_changed),  NULL);
 
   /* general signals */
   g_signal_connect (G_OBJECT (parent), "response", G_CALLBACK (cb_dialog_response), NULL);
@@ -957,7 +868,7 @@ update_list_something (GtkWidget *tree_view, const gchar *theme)
     }
 }
 
-void
+static void
 update_color_buttons_from_string (gchar *color_scheme)
 {
   GdkColor color_scheme_colors[6];
@@ -1025,6 +936,7 @@ update_color_buttons_from_string (gchar *color_scheme)
   widget = WID ("selected_bg_colorbutton");
   gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), &color_scheme_colors[5]);
 
+
 }
 
 void
@@ -1032,11 +944,10 @@ gnome_theme_details_update_from_gconf (void)
 {
   GConfClient *client;
   GladeXML *dialog;
-  GtkWidget *tree_view, *widget;
+  GtkWidget *tree_view;
   gchar *theme = NULL;
   GnomeWindowManager *window_manager = NULL;
   GnomeWMSettings wm_settings;
-  gboolean use_custom_colors = FALSE;
 
   gnome_theme_details_init ();
 
@@ -1049,6 +960,7 @@ gnome_theme_details_update_from_gconf (void)
   tree_view = WID ("control_theme_treeview");
   theme = gconf_client_get_string (client, GTK_THEME_KEY, NULL);
   update_list_something (tree_view, theme);
+  update_color_scheme_tab (theme);
   g_free (theme);
 
   tree_view = WID ("window_theme_treeview");
@@ -1063,26 +975,28 @@ gnome_theme_details_update_from_gconf (void)
   update_list_something (tree_view, theme);
   g_free (theme);
 
-
-  /* color preferences */
-
+  /* update colour scheme tab */
   theme = gconf_client_get_string (client, COLOR_SCHEME_KEY, NULL);
-  if (theme)
-  {
-    use_custom_colors = strcmp ("", theme);
-    update_color_buttons_from_string (theme);
-    g_free (theme);
-  }
-
-  widget = WID ("enable_custom_colors_checkbutton");
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), use_custom_colors);
-
-  widget = WID ("saved_color_schemes_hbox");
-  gtk_widget_set_sensitive (widget, use_custom_colors);
-
-  widget = WID ("color_scheme_table");
-  gtk_widget_set_sensitive (widget, use_custom_colors);
-
+  update_color_buttons_from_string (theme);
+  g_free (theme);
 
   g_object_unref (client);
 }
+
+static void
+color_scheme_changed (GObject    *settings,
+                      GParamSpec *pspec,
+                      gpointer    data)
+{
+  GConfClient *client;
+  gchar *theme;
+
+  client = gconf_client_get_default ();
+  theme = gconf_client_get_string (client, COLOR_SCHEME_KEY, NULL);
+  if (theme == NULL || strcmp (theme, "") == 0)
+    g_object_get (settings, "gtk-color-scheme", &theme, NULL);
+
+  update_color_buttons_from_string (theme);
+  g_free (theme);
+  g_object_unref (client);
+}
